import os,time
from PyQt5.Qt import QSystemTrayIcon,QIcon,QMenu,QAction,QFont,QCursor
from PyQt5.QtCore import QEvent, QSize, pyqtSlot,pyqtSignal,Qt,QRect
from PyQt5.QtWidgets import QLabel, QPushButton, QHBoxLayout,QFrame,\
    QApplication, QWidget,QVBoxLayout,QRadioButton,QCheckBox,QDesktopWidget
class Window(QWidget):
    Signal = pyqtSignal(bool)
    def __init__(self):
        super(Window,self).__init__()
        self.setWindowTitle("关闭")
        self.setupUi()
        self.show()
        self.resize(393, 238)
        self.center()
    def setupUi(self):
        self.widget = QWidget()
        self.Layout = QVBoxLayout(self.widget)
        self.Layout.setContentsMargins(0,0,0,0)
        self.setLayout(self.Layout)
        self.setWindowFlag(Qt.Tool)
        self.main_widget = QWidget()
        self.Layout.addWidget(self.main_widget)
        self.closed = QPushButton(self.main_widget)
        self.closed.setGeometry(QRect(352, 10, 21, 21))
        self.closed.setObjectName("close")
        self.closed.setStyleSheet('''
        QPushButton{background:#FF6347;border-radius:10px}
        QPushButton:hover{background:red}''')
        self.close_label = QLabel(self.main_widget)
        self.close_label.setText("关闭")
        self.close_label.setGeometry(QRect(10, 10, 61, 31))
        self.close_label.setFont(QFont("Roman times",11))
        self.line = QFrame(self.main_widget)
        self.line.setGeometry(QRect(10, 30, 371, 20))
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.closer = QLabel(self.main_widget)
        self.closer.setGeometry(QRect(0, 50, 161, 30))
        self.closer.setText("是否确认关闭")
        self.closer.setFont(QFont("Roman times",15,QFont.Bold))
        self.close1 = QRadioButton(self.main_widget)
        self.close1.setGeometry(QRect(90, 100, 130, 30))
        self.close1.setText("直接关闭")
        self.close1.setChecked(True)
        self.close1.setFont(QFont("Roman times",15,QFont.Bold))
        self.close2 = QRadioButton(self.main_widget)
        self.close2.setGeometry(QRect(90, 140, 161, 30))
        self.close2.setText("最小化托盘")
        self.close2.setFont(QFont("Roman times",15,QFont.Bold))
        self.remember = QCheckBox(self.main_widget)
        self.remember.setGeometry(QRect(10, 190, 110, 30))
        self.remember.setText("记住选择")
        self.remember.setFont(QFont("Roman times",12,QFont.Bold))
        self.ok_btn = QPushButton(self.main_widget)
        self.ok_btn.setGeometry(QRect(200, 200, 80, 30))
        self.ok_btn.setText("确认")
        self.ok_btn.clicked.connect(self.on_clicked)
        self.exit_out = QPushButton(self.main_widget)
        self.exit_out.setGeometry(QRect(300, 200, 80, 30))
        self.exit_out.setText("取消")
        self.exit_out.clicked.connect(self.close)
        self.setWindowFlags(Qt.FramelessWindowHint)
    def mousePressEvent(self, event):
        try:
            if event.button()==Qt.LeftButton:
                self.m_flag=True
                self.m_Position=event.globalPos()-self.pos()
                event.accept()
                self.setCursor(QCursor(Qt.OpenHandCursor))
        except:
            pass
    def mouseMoveEvent(self, QMouseEvent):
        try:
            if Qt.LeftButton and self.m_flag:
                self.move(QMouseEvent.globalPos()-self.m_Position)
                QMouseEvent.accept()
        except:
            pass
    def mouseReleaseEvent(self, QMouseEvent):
        try:
            self.m_flag=False
            self.setCursor(QCursor(Qt.ArrowCursor))
        except:
            pass
    def on_clicked(self):
        self.close()
        if self.close1.isChecked():
            self.chooses = True
        elif self.close2.isChecked():
            self.chooses = False
        self.Signal.emit(self.chooses)
        self.choose()
    def center(self):
        self.qr = self.frameGeometry()
        self.cp = QDesktopWidget().availableGeometry().center()
        self.qr.moveCenter(self.cp)
        self.move(self.qr.topLeft())
    def choose(self):
        if self.remember.isChecked():
            if os.path.exists('remember'):
                os.popen(r'attrib -s -r -h remember')
                time.sleep(0.5)
            with open('remember',"w") as f:
                f.write("{}".format(self.chooses))
                os.popen(r'attrib +s +r +h remember')


import os

# 这些图片路径是相对main.py的, 而不是相对本py文件的
# 下面这些图片资源是TitleBar的
IMG_MIN_NORM = "minsize.png"
IMG_MIN_HOVER = "minsizehover.png"
IMG_MIN_PRESS = "minsizepress.png"
IMG_RET_NORM = "retsize.png"
IMG_RET_HOVER = "retsizehover.png"
IMG_RET_PRESS = "retsizepress.png"
IMG_RESIZE_NORM = "resize.png"
IMG_RESIZE_HOVER = "resizehover.png"
IMG_RESIZE_PRESS = "resizepress.png"
IMG_CLOSE_NORM = "close.png"
IMG_CLOSE_HOVER = "closehover.png"
IMG_CLOSE_PRESS = "closepress.png"
ICON_NORM = "icon.jpg"

# 资源路径: 相对路径没找到就去安装路径下找
IMAGE_ROOT = './beautifyUi/'
if not os.path.exists(IMAGE_ROOT):
    IMAGE_ROOT = (os.path.split(__file__)[0] + "\\beautifyUi\\").replace('\\', '/')



import win32gui
import win32con
import os,time
from PyQt5.Qt import QSizePolicy,QSystemTrayIcon,QIcon,QMenu,QAction,QFont
from PyQt5.QtCore import QEvent, QSize, pyqtSlot,pyqtSignal,Qt,QRect
from PyQt5.QtWidgets import QLabel, QPushButton, QHBoxLayout,QMessageBox,\
    QApplication, QWidget,QVBoxLayout,QRadioButton,QCheckBox,QDesktopWidget
from . import CloseWindow
from .resourse_cfg import *


class Titlebar(QWidget):
    # 默认基础样式参数
    TITLE_TEXT_COLOR = "white"
    BGD_COLOR = "#28AAAA"
    TITLEBAR_HEIGHT = 30
    ICON_SIZE = QSize(20, 20)
    MIN_BUTT_SIZE = QSize(27, 22)
    RET_BUTT_SIZE = QSize(27, 22)
    CLOSE_BUTT_SIZE = QSize(27, 22)
    TITLE_LABEL_NAME = "Titlebar_titleLabel"
    BACKGROUND_LABEL_NAME = "Titlebar_backgroundLabel"
    MIN_BUTT_NAME = "Titlebar_minimizeButton"
    RET_BUTT_NAME = "Titlebar_maximizeButton"
    CLOSE_BUTT_NAME = "Titlebar_closeButton"
    THEME_IMG_DIR = 'default'

    def __init__(self, parent, icon_name):
        super(Titlebar, self).__init__(parent)
        self.parentwidget = parent
        self.setFixedHeight(Titlebar.TITLEBAR_HEIGHT)
        self.icon_name = icon_name
        self.m_pBackgroundLabel = QLabel(self)
        self.m_pIconLabel = QLabel(self)
        self.m_pTitleLabel = QLabel(self)
        self.m_pMinimizeButton = QPushButton(self)
        self.m_pReturnButton = QPushButton(self)
        self.m_pCloseButton = QPushButton(self)
        self.m_pIconLabel.setFixedSize(Titlebar.ICON_SIZE)
        self.m_pIconLabel.setScaledContents(True)
        self.m_pTitleLabel.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.m_pBackgroundLabel.setObjectName(Titlebar.BACKGROUND_LABEL_NAME)
        # 三大金刚按钮大小
        self.m_pReturnButton.setFixedSize(Titlebar.RET_BUTT_SIZE)
        self.m_pMinimizeButton.setFixedSize(Titlebar.MIN_BUTT_SIZE)
        self.m_pCloseButton.setFixedSize(Titlebar.CLOSE_BUTT_SIZE)
        # 统一设置ObjName
        self.m_pTitleLabel.setObjectName(Titlebar.TITLE_LABEL_NAME)
        self.m_pBackgroundLabel.resize(self.parentwidget.width(), Titlebar.TITLEBAR_HEIGHT)
        self.m_pReturnButton.setObjectName(Titlebar.RET_BUTT_NAME)
        self.m_pMinimizeButton.setObjectName(Titlebar.MIN_BUTT_NAME)
        self.m_pCloseButton.setObjectName(Titlebar.CLOSE_BUTT_NAME)
        # 布局
        pLayout = QHBoxLayout(self)
        pLayout.addWidget(self.m_pIconLabel)
        pLayout.addSpacing(5)
        pLayout.addWidget(self.m_pTitleLabel)
        pLayout.addWidget(self.m_pReturnButton)
        pLayout.addWidget(self.m_pMinimizeButton)
        pLayout.addWidget(self.m_pCloseButton)
        pLayout.setSpacing(0)
        pLayout.setContentsMargins(5, 0, 5, 0)
        self.setLayout(pLayout)
        # 信号连接
        self.m_pReturnButton.clicked.connect(self.__slot_onclicked)
        self.m_pMinimizeButton.clicked.connect(self.__slot_onclicked)
        self.m_pCloseButton.clicked.connect(self.__slot_onclicked)
        # 置中
        self.center()
        # 设置默认样式(bar的字颜色和背景颜色)
        # self.setTitleBarStyle(Titlebar.BGD_COLOR, Titlebar.TITLE_TEXT_COLOR)

    def setTitleBarStyle(self, backgroundColor, textColor):
        # 标题字体颜色
        self.m_pTitleLabel.setStyleSheet("font-size:13px;margin-bottom:0px;color:%s" % (textColor))
        # 标题栏背景颜色
        self.m_pBackgroundLabel.setStyleSheet("background:%s" % (backgroundColor))

    def mousePressEvent(self, e):
        """
        使窗口能被拖动
        :param e:
        :return:
        """
        win32gui.ReleaseCapture()
        pWindow = self.window()
        if pWindow.isWindow():
            win32gui.SendMessage(pWindow.winId(), win32con.WM_SYSCOMMAND, win32con.SC_MOVE + win32con.HTCAPTION, 0)
        e.ignore()

    def eventFilter(self, object, e):
        if e.type() == QEvent.WindowTitleChange:
            if object != None:
                self.m_pTitleLabel.setText(object.windowTitle())
                return True
        if e.type() == QEvent.WindowIconChange:
            if object != None:
                icon = object.windowIcon()
                self.m_pIconLabel.setPixmap(icon.pixmap(self.m_pIconLabel.size()))
                return True
        if e.type() == QEvent.Resize:
            self.__setTitleBarSize(self.parentwidget.width())
            return True
        return QWidget.eventFilter(self, object, e)

    @pyqtSlot()
    def __slot_onclicked(self):
        pButton = self.sender()
        pWindow = self.window()
        if pWindow.isWindow():
            if pButton.objectName() == Titlebar.RET_BUTT_NAME:
                self.delete()
            if pButton.objectName() == Titlebar.MIN_BUTT_NAME:
                pWindow.showMinimized()
            elif pButton.objectName() == Titlebar.CLOSE_BUTT_NAME:
                self.close()
                
    def center(self):
        self.qr = self.frameGeometry()
        self.cp = QDesktopWidget().availableGeometry().center()
        self.qr.moveCenter(self.cp)
        self.move(self.qr.topLeft())

    def delete(self):
        if os.path.exists('remember'):
            os.popen(r'attrib -s -r -h remember | del/s/q remember')
            QMessageBox.about(self,"成功","取消记住选择")

    def closeEvent(self,event):
        """
        关闭弹窗弹出
        :param event:
        :return:
        """
        if os.path.exists('remember'):
            with open('remember', 'r') as f:
                result = f.read()
            if result == "True":
                self.closer(True)
            else:
                event.ignore()
                self.closer(False)
            return
        event.ignore()
        self.newWindow_Close = CloseWindow.Window()
        self.newWindow_Close.Signal.connect(self.closer)

    def closer(self,choose):
        """
        托盘界面设置
        :param choose:
        :return:
        """
        if choose == True:
            QApplication.instance().quit()
        elif choose == False:
            self.window().hide()
            self.tray = QSystemTrayIcon(self)
            if os.path.exists(ICON_NORM):
                self.icon = QIcon(ICON_NORM)
            else:
                self.icon = QIcon((os.path.split(__file__)[0] + "\\beautifyUi\\").replace('\\', '/')+ICON_NORM)
            self.tray.setIcon(self.icon)
            self.tray.activated.connect(self.TuoPanEvent)
            self.tray.setToolTip(self.icon_name)
            self.tray_menu = QMenu(QApplication.desktop())
            self.RestoreAction = QAction(u'还原',self,triggered=self.window().show)
            self.QuitAction = QAction(u'退出',self,triggered=QApplication.instance().quit)
            self.tray_menu.addAction(self.RestoreAction)
            self.tray_menu.addAction(self.QuitAction)
            self.tray.setContextMenu(self.tray_menu)
            self.tray.show()
            self.tray.showMessage(self.icon_name, '托盘在这！', icon=1)
        else:
            QMessageBox.warning(self,"警告","系统出现问题,\n请稍后再试")

    def TuoPanEvent(self,reason):
        """
        托盘两键设置
        :param reason:
        :return:
        """
        qApp = QApplication.instance()
        if reason == QSystemTrayIcon.DoubleClick:
            if self.isMinimized() or not self.isVisible():
                self.showNormal()
                self.activateWindow()
            else:
                self.showMinimized()

    def __setTitleBarSize(self, width):
        self.m_pBackgroundLabel.resize(width, Titlebar.TITLEBAR_HEIGHT)



import json
import os
from random import choice
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import qApp

from . import WindowWithTitleBar
from . import simple_qss

"""
注意:托盘图标默认名称为icon.jpg,改变该图标可在运行目录下放置该图片
- 使用示例:
    from cureUi import cure
    app = QApplication(sys.argv)
    win = cure.Windows(Examples(), 'tray name', 'pink', 'program name', 'myicon.ico')
    sys.exit(app.exec_())
"""
RESOURCE_DIR = 'beautifyUi'

def Windows(mainWidget, icon_name, theme=None, title='QCureWindow', ico_path=''):
    """
    快速创建彩色窗 (带TitleBar)
    :param mainWidget:
    :param theme:
    :param title:
    :param ico_path:
    :return:
    """
    theme = set_theme(theme)
    coolWindow = WindowWithTitleBar.WindowWithTitleBar(mainWidget, icon_name)
    coolWindow.setWindowTitle(title)
    coolWindow.setWindowIcon(QIcon(ico_path))
    coolWindow.show()
    setTheme(theme)
    return coolWindow


def set_theme(theme):
    """
    判断theme值是否为随机返回
    :param theme:
    :return:
    """
    if theme == True:
        dirList = []
        if os.path.isfile(RESOURCE_DIR):
            path = RESOURCE_DIR
        else:
            path = (os.path.split(__file__)[0] + '\\' + RESOURCE_DIR + '\\').replace('\\', '/')
        for i in os.listdir(path):
            if os.path.isdir(path + i):
                dirList.append(i)
        if len(dirList) > 0:
            return choice(dirList)
        else:
            return theme
    else:
        return theme


def setTheme(theme):
    """
    根据theme.json设置主题的qss (只改样式不加Titlebar)
    :param theme:
    :return:
    """
    THEME_FILE = RESOURCE_DIR + '/theme.json'
    if os.path.isfile(THEME_FILE):
        path = THEME_FILE
    else:
        path = (os.path.split(__file__)[0] + '\\' + THEME_FILE).replace('\\', '/')
    tDict = json.load(open(path))
    # theme.json的theme的优先级比setTheme中的theme的优先级高
    configTheme = tDict.get('theme')
    if configTheme is None or configTheme == '' or tDict.get(configTheme) is None:
        colorDict = tDict.get(theme)
    else:
        colorDict = tDict.get(configTheme)
    if colorDict is None:
        qss = simple_qss.getDefaultQss()
    else:
        qss = simple_qss.getQss(colorDict['fontLight'], colorDict['fontDark'], colorDict['normal'], colorDict['light'],
                                colorDict['deep'], colorDict['disLight'], colorDict['disDark'], theme)
    qApp.setStyleSheet(qss)



from . import qss_getter as Qss
from .Titlebar import Titlebar
from .resourse_cfg import *


def getDefaultQss():
    """
    默认主题
    :return:
    """
    # fontLight, fontDark, normal, light, deep, disLight, disDark
    return getQss(Qss.WHITE, Qss.DEEPBLUEGREEN, Qss.BLUEGREEN, Qss.LIGHTGREEN, Qss.DARKBLUEGREEN, Qss.LIGHTGRAY, Qss.GRAY, "default")


def getQss(fontLight, fontDark, normal, light, deep, disLight, disDark, themeImgDir):
    """
    通用组件的Qss + cureBar的Qss
    :param fontLight:
    :param fontDark:
    :param normal:
    :param light:
    :param deep:
    :param disLight:
    :param disDark:
    :param themeImgDir:
    :return:
    """
    themeImgDir = themeImgDir if os.path.isdir(IMAGE_ROOT + themeImgDir) else 'default'
    qss = str()
    qss += __getWidgetsQss(fontLight, fontDark, normal, light, deep, disLight, disDark, themeImgDir)
    qss += __getcureQss(fontLight, deep, fontLight, themeImgDir)
    return qss


def __getWidgetsQss(fontLight, fontDark, normal, light, deep, disLight, disDark, themeImgDir):
    """
    通用组件(Widgets)的Qss
    :param fontLight:
    :param fontDark:
    :param normal:
    :param light:
    :param deep:
    :param disLight:
    :param disDark:
    :param themeImgDir:
    :return:
    """
    qss = str()
    qss += Qss.getFontQss("微软雅黑", fontDark)
    qss += Qss.getPushButtonQss(normal, fontLight, light, normal, disLight, fontLight, disDark, disLight)
    qss += Qss.getPlaineditQss(disLight, normal)
    qss += Qss.getTextBrowerQss(disLight, normal)
    qss += Qss.getLineeditQss(disLight, normal)
    qss += Qss.getComboxQss(fontLight, disLight, normal, IMAGE_ROOT + themeImgDir + "/" + "down_arrow.png")
    img_norm = IMAGE_ROOT + themeImgDir + "/" + "radio_normal.png"
    img_down = IMAGE_ROOT + themeImgDir + "/" + "radio_down.png"
    img_hover = IMAGE_ROOT + themeImgDir + "/" + "radio_hoverUncheck.png"
    img_downhover = IMAGE_ROOT + themeImgDir + "/" + "radio_hoverCheck.png"
    qss += Qss.getRadioButtonQss(img_norm, img_down, img_hover, img_downhover)
    img_norm = IMAGE_ROOT + themeImgDir + "/" + "checkbox_normal.png"
    img_down = IMAGE_ROOT + themeImgDir + "/" + "checkbox_down.png"
    img_hover = IMAGE_ROOT + themeImgDir + "/" + "checkbox_hoverUncheck.png"
    img_downhover = IMAGE_ROOT + themeImgDir + "/" + "checkbox_hoverCheck.png"
    qss += Qss.getCheckBoxQss(img_norm, img_down, img_hover, img_downhover)
    qss += Qss.getTabWidgetQss(normal, fontLight, normal)
    qss += Qss.getSliderQss(normal, fontLight, normal)
    qss += Qss.getScrollbarQss(normal)
    return qss


def __getcureQss(barTextColor, barColor, winBgdColor, themeImgDir):
    """
    TitleBar+Window的Qss
    :param barTextColor: 文字颜色
    :param barColor: bar主体颜色
    :param winBgdColor: 主体窗口背景颜色
    :param themeImgDir: 主题名(作用主要是为了找按钮图片)
    :return: qss
    """
    Titlebar.THEME_IMG_DIR = themeImgDir
    qss = str()
    qss += "Titlebar QLabel#%s{font-size:13px;margin-bottom:0px;color:%s;}" % (Titlebar.TITLE_LABEL_NAME, barTextColor)
    qss += "Titlebar QLabel#%s{background:%s;}" % (Titlebar.BACKGROUND_LABEL_NAME, barColor)
    # 三大金刚键的图片设置 (最大化恢复正常大小的图片设置只能在Title的onclick中设置)
    qss += "Titlebar QPushButton#%s{background:transparent; background-image:url(%s); border:none}" % \
           (Titlebar.RET_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_RET_NORM)
    qss += "Titlebar QPushButton#%s:hover{background:transparent; background-image:url(%s)}" % \
           (Titlebar.RET_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_RET_HOVER)
    qss += "Titlebar QPushButton#%s:pressed{background:transparent; background-image:url(%s)}" % \
           (Titlebar.RET_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_RET_PRESS)
    qss += "Titlebar QPushButton#%s:disabled{background:transparent; background-image:url(%s)}" % \
           (Titlebar.RET_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_RET_PRESS)
    qss += "Titlebar QPushButton#%s{background:transparent; background-image:url(%s); border:none}" % \
           (Titlebar.MIN_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_MIN_NORM)
    qss += "Titlebar QPushButton#%s:hover{background:transparent; background-image:url(%s)}" % \
           (Titlebar.MIN_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_MIN_HOVER)
    qss += "Titlebar QPushButton#%s:pressed{background:transparent; background-image:url(%s)}" % \
           (Titlebar.MIN_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_MIN_PRESS)
    qss += "Titlebar QPushButton#%s:disabled{background:transparent; background-image:url(%s)}" % \
           (Titlebar.MIN_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_MIN_PRESS)
    qss += "Titlebar QPushButton#%s{background:transparent; background-image:url(%s); border:none}" % \
           (Titlebar.CLOSE_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_CLOSE_NORM)
    qss += "Titlebar QPushButton#%s:hover{background:transparent; background-image:url(%s)}" % \
           (Titlebar.CLOSE_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_CLOSE_HOVER)
    qss += "Titlebar QPushButton#%s:pressed{background:transparent; background-image:url(%s)}" % \
           (Titlebar.CLOSE_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_CLOSE_PRESS)
    qss += "Titlebar QPushButton#%s:disabled{background:transparent; background-image:url(%s)}" % \
           (Titlebar.CLOSE_BUTT_NAME, IMAGE_ROOT + themeImgDir + "/" + IMG_CLOSE_PRESS)
    # Window窗口内底色+外围描边
    qss += "WindowWithTitleBar{background:%s;border:3px solid %s}" % (winBgdColor, barColor)
    return qss



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="QcureUi",
    version="0.0.2",
    author="Lux",
    author_email="1223411083@qq.com",
    description="Used for pyqt5 beautification",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires= [
        'pywin32'
    ],
    project_urls={
        'Blog': 'https://blog.csdn.net/qq_45414559/article/details/105560090',
    },
)
